import pytest

import numpy as np

from take_confusion_matrix import MetricsCalculator


def test_different_size_arrays_raises_value_error():
    mc = MetricsCalculator([0, 1])

    y_true = [0, 1, 0, 1]
    y_pred = [0, 0, 0, 0]

    mc.compute_matrix(y_true, y_pred)

    with pytest.raises(ValueError):
        mc.generate_confusion_matrix(labels=[1])


def test_different_size_labels_raises_value_error():
    mc = MetricsCalculator([0, 1])

    with pytest.raises(ValueError):
        mc.compute_matrix([0, 1], [0, 1, 2])


def test_different_type_arrays_raises_type_error():
    mc = MetricsCalculator([0, 1])

    with pytest.raises(TypeError):
        mc.compute_matrix(0, 0)


def test_different_type_labels_raises_type_error():
    with pytest.raises(TypeError):
        MetricsCalculator(0)


def test_different_type_labels_and_arrays_raises_type_error():
    mc = MetricsCalculator([0, 1])
    y_true = ["class_1", "class_1"]

    with pytest.raises(ValueError):
        mc.compute_matrix(y_true, y_true)


def test_labels_with_invalid_size_raises_value_error():
    with pytest.raises(ValueError):
        MetricsCalculator([])


def test_different_params_type_raises_type_error():
    mc = MetricsCalculator([0, 1])

    with pytest.raises(TypeError):
        mc.generate_confusion_matrix([])
        mc.generate_confusion_matrix(with_labels=[])
        mc.generate_confusion_matrix(labels=0)



import numpy as np


def check_value_mapped(mapper: dict, values):
    """Check that all values passed were mapped

    Checks whether the values passed were previously mapped.

    Parameters
    ----------
    mapper : dict
        Dictionary with the values previously mapped.

    values : 1d array-like
        Values to be validated.

    Raises
    -------
    ValueError
        If the parameter's value violates the given bounds.
    """

    for value in values:
        if not mapper.get(value, False):
            raise ValueError(f"Found value `{value}` while "
                             f"the expected ones were {list(mapper.keys())}.")


def check_array_size(x: object, name: str, min_size: [int, float]):
    """Validate scalar parameter size.

    Parameters
    ----------
    x : object
        The scalar parameter to validate.

    name : str
        The name of the parameter to be printed in error messages.

    min_size : int or float
        The minimum valid size the parameter can take. 

    Raises
    -------
    TypeError
        If the parameter's type does not match the desired type.

    ValueError
        If the parameter's value violates the given bounds.
    """
    size = len(x)
    if size < min_size:
        raise ValueError(f"Size of `{name}` = {size}"
                         f", must be >= {min_size}")



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as fr:
    requirements = fr.read().splitlines()

setuptools.setup(
    name="TakeConfusionMatrix",
    version="0.0.3",
    author="Cecília Assis",
    author_email="cecilia.assis@take.net",
    description="TakeConfusionMatrix is a tool for batched metrics calculations",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://curupira.visualstudio.com/DefaultCollection/Data%20Analytics%20%28DA%29/_git/TakeConfusionMatrix",
    maintainer="D&A Team",
    maintainer_email="analytics.ped@take.net",
    packages=setuptools.find_packages(),
    install_requires=requirements,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
)



import pytest

import numpy as np

from take_confusion_matrix import MetricsCalculator


def test_matrix_for_two_unchaging_labels_returns_expected_results():
    labels = [0, 1]

    mc = MetricsCalculator(labels)

    y_true = [0, 1, 0, 1]
    y_pred = [0, 0, 0, 0]

    mc.compute_matrix(y_true, y_pred)

    expected_matrix = [[2, 2],
                       [0, 0]]
    actual_matrix = mc.generate_confusion_matrix().values

    assert (expected_matrix == actual_matrix).all()


def test_matrix_for_three_unchanging_labels_returns_expected_results():
    labels = [0, 1, 2]

    mc = MetricsCalculator(labels)

    y_true = [2, 0, 2, 2, 0, 1]
    y_pred = [0, 0, 2, 2, 0, 2]

    mc.compute_matrix(y_true, y_pred)

    expected_matrix = [[2, 0, 1],
                       [0, 0, 0],
                       [0, 1, 2]]
    actual_matrix = mc.generate_confusion_matrix().values

    assert (expected_matrix == actual_matrix).all()


def test_matrix_for_three_changing_labels_returns_expected_results():
    labels = [0, 1, 2]

    mc = MetricsCalculator(labels)

    y_true = [0, 1, 0, 1]
    y_pred = [0, 0, 0, 0]

    mc.compute_matrix(y_true, y_pred)

    y_true = [0, 2, 0, 2]
    y_pred = [0, 0, 0, 0]

    mc.compute_matrix(y_true, y_pred)

    expected_matrix = [[4, 2, 2],
                       [0, 0, 0],
                       [0, 0, 0]]
    actual_matrix = mc.generate_confusion_matrix().values

    assert (expected_matrix == actual_matrix).all()


def test_matrix_batched_computation_returns_expected_results():
    labels = [0, 1, 2]

    mc = MetricsCalculator(labels)

    times = 10

    y_true = [2, 0, 2, 2, 0, 1]
    y_pred = [0, 0, 2, 2, 0, 2]

    for i in range(times):
        mc.compute_matrix(y_true, y_pred)

    expected_matrix = [[2 * times, 0,         1 * times],
                       [0,         0,         0],
                       [0,         1 * times, 2 * times]]
    actual_matrix = mc.generate_confusion_matrix().values

    assert (expected_matrix == actual_matrix).all()


def test_diagonal_matrix_computation_returns_expected_results():
    labels = [0, 1, 2]

    mc = MetricsCalculator(labels)

    times = 10

    y_true = [0, 0, 1, 1, 2, 2]
    y_pred = [0, 0, 1, 1, 2, 2]

    for i in range(times):
        mc.compute_matrix(y_true, y_pred)

    expected_matrix = [[2 * times, 0,         0],
                       [0,         2 * times, 0],
                       [0,         0,         2 * times]]
    actual_matrix = mc.generate_confusion_matrix().values

    assert (expected_matrix == actual_matrix).all()


def test_string_labels_matrix_computation_returns_expected_results():
    labels = ["class_0", "class_1"]

    mc = MetricsCalculator(labels)

    y_true = ["class_0", "class_1", "class_0", "class_1"]
    y_pred = ["class_0", "class_0", "class_0", "class_0"]

    mc.compute_matrix(y_true, y_pred)

    expected_matrix = [[2, 2],
                       [0, 0]]
    actual_matrix = mc.generate_confusion_matrix().values

    assert (expected_matrix == actual_matrix).all()


def test_matrix_with_user_specified_labels_retuns_uses_labels():
    labels = [0, 1]

    mc = MetricsCalculator(labels)

    y_true = [0, 1, 0, 1]
    y_pred = [0, 0, 0, 0]

    mc.compute_matrix(y_true, y_pred)

    labels = ["class_0", "class_1"]
    matrix = mc.generate_confusion_matrix(labels=labels)

    assert (matrix.index.values == labels).all()
    assert (matrix.columns == labels).all()
    assert matrix["class_0"]["class_0"] == 2


def test_matrix_without_labels_returns_ndarray():
    labels = [0, 1]

    mc = MetricsCalculator(labels)

    y_true = [0, 1, 0, 1]
    y_pred = [0, 0, 0, 0]

    mc.compute_matrix(y_true, y_pred)

    matrix = mc.generate_confusion_matrix(with_labels=False)

    assert isinstance(matrix, np.ndarray)


def test_metrics_computation_returns_expected_results():
    labels = [0, 1, 2]

    mc = MetricsCalculator(labels)

    y_true = [2, 0, 2, 2, 0, 1]
    y_pred = [0, 0, 2, 2, 0, 2]

    mc.compute_matrix(y_true, y_pred)

    expected_dict = {"0": {"precision": 0.6666666666666666,
                           "recall": 1.0,
                           "f1-score": 0.8},
                     "1": {"precision": 0.0,
                           "recall": 0.0,
                           "f1-score": 0.0},
                     "2": {"precision": 0.6666666666666666,
                           "recall": 0.6666666666666666,
                           "f1-score": 0.6666666666666666},
                     "accuracy": 0.6666666666666666}
    actual_metrics = mc.generate_metrics()

    assert actual_metrics == expected_dict


def test_matrix_returns_normalized_results():
    labels = [0, 1, 2]

    mc = MetricsCalculator(labels)

    y_true = [2, 0, 2, 2, 0, 1]
    y_pred = [0, 0, 2, 2, 0, 2]

    mc.compute_matrix(y_true, y_pred)

    expected_matrix = [[0.66666667, 0.,         0.33333333],
                       [0.,         0.,         0.],
                       [0.,         0.33333333, 0.66666667]]
    actual_matrix = mc.generate_confusion_matrix(normalize=True,
                                                 with_labels=False)

    assert expected_matrix == pytest.approx(actual_matrix)
